<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-montonio-m1
 * @copyright  Copyright (c) 2021 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

/**
 * @inheritdoc
 */
class Montonio_Gateway_Helper_Payment extends Mage_Payment_Helper_Data
{

    /**
     * @inheritdoc
     */
    public function getInfoBlock(Mage_Payment_Model_Info $info)
    {
        $blockType = $info->getMethodInstance()->getInfoBlockType();
        if ($this->getLayout()) {
            $block = $this->getLayout()->createBlock($blockType);
        } else {
            $className = Mage::getConfig()->getBlockClassName($blockType);
            $block = new $className;
        }
        $block->setInfo($info);
        return $block;
    }

    /**
     * @inheritdoc
     */
    public function getMethodInstance($code)
    {
        //this needs helping out
        if (strstr($code, 'montonio_payments_')) {
            $code = 'montonio_payments';
        }
        $key = self::XML_PATH_PAYMENT_METHODS.'/'.$code.'/model';
        $class = Mage::getStoreConfig($key);
        return Mage::getModel($class);
    }

    /**
     * @inheritdoc
     */
    public function getStoreMethods($store = null, $quote = null)
    {
        $res = [];
        foreach ($this->getPaymentMethods($store) as $code => $methodConfig) {

            //this needs helping out
            if (strstr($code, 'montonio_payments_')) {
                $prefix = self::XML_PATH_PAYMENT_METHODS . '/montonio_payments/';
            } else {
                $prefix = self::XML_PATH_PAYMENT_METHODS . '/' . $code . '/';
            }

            if (!$model = Mage::getStoreConfig($prefix . 'model', $store)) {
                continue;
            }
            $methodInstance = Mage::getModel($model);
            if (!$methodInstance) {
                continue;
            }

            //remove from list if not grouped
            if ($methodInstance->getCode() == 'montonio_payments' &&
                !strstr($code, 'montonio_payments_')
            ) {
                if ((int)$methodInstance->getConfigData('grouped') === 0) {
                    continue;
                }
            }

            $methodInstance->setStore($store);
            if (!$methodInstance->isAvailable($quote)) {
                /* if the payment method cannot be used at this time */
                continue;
            }
            $sortOrder = (int)$methodInstance->getConfigData('sort_order', $store);
            $methodInstance->setSortOrder($sortOrder);
            $methodInstance->setCode($code);

            $res[] = $methodInstance;
        }

        usort($res, [$this, '_sortMethods']);
        return $res;
    }

    /**
     * @inheritdoc
     */
    public function getPaymentMethods($store = null)
    {

        $result     = parent::getPaymentMethods($store);

        $method     = Mage::getModel('montonio/method_payments');
        $enabled    = $method->getConfigData('active');
        $methodCode = $method->getCode();

        //add our virtual codes to config list based on our actual shared class
        if ($enabled && !empty($result[$methodCode])) {

            $availableMethods       = $method->getAvailableMethods();
            $isGrouped              = $method->getConfigData('grouped');

            if ($isGrouped) {
                $availableMethods   = ['grouped' => '0'];
            }

            foreach ($availableMethods as $code => $data) {
                if ($code == 'grouped') {
                    $subMethodCode = $methodCode;
                } else {
                    $subMethodCode = $methodCode. '_' .$code;
                }
                $result[$subMethodCode]=$result[$methodCode];
            }

        }

        return $result;
    }
}
